from datetime import timedelta
from unittest.mock import Mock

from django.utils import timezone
from tiers.models import Tier
from tiers.tests.factories import TierFactory
from tiers.admin import ActiveTierFilter


def test_active_tier_filter_lookup_choices():
    filter = ActiveTierFilter(Mock(), {}, Mock(), Mock())
    assert filter.lookups(Mock(), Mock()) == [
        ['active', 'Active Tiers'],
        ['inactive', 'Inactive Tiers'],
    ]


def test_active_tier_filter_queryset(db):
    past_date = timezone.now() - timedelta(days=100)
    future_date = timezone.now() + timedelta(days=100)
    expired_inactive_tier = TierFactory.create(tier_enforcement_exempt=False, tier_expires_at=past_date)
    exempted_active_tier = TierFactory.create(tier_enforcement_exempt=True, tier_expires_at=past_date)
    active_tier = TierFactory.create(tier_enforcement_exempt=True, tier_expires_at=future_date)
    tiers = Tier.objects.all()

    active_filter = ActiveTierFilter(Mock(), {'tier_status': 'active'}, Mock(), Mock())
    assert active_tier in active_filter.queryset(Mock(), tiers), 'Should only return active tiers'
    assert exempted_active_tier in active_filter.queryset(Mock(), tiers), 'Should only return active tiers'
    assert expired_inactive_tier not in active_filter.queryset(Mock(), tiers), 'Should only return active tiers'

    inactive_filter = ActiveTierFilter(Mock(), {'tier_status': 'inactive'}, Mock(), Mock())
    assert active_tier not in inactive_filter.queryset(Mock(), tiers), 'Should only return inactive tiers'
    assert exempted_active_tier not in inactive_filter.queryset(Mock(), tiers), 'Should only return inactive tiers'
    assert expired_inactive_tier in inactive_filter.queryset(Mock(), tiers), 'Should only return inactive tiers'

    all_filter = ActiveTierFilter(Mock(), {}, Mock(), Mock())
    assert len(all_filter.queryset(Mock(), tiers)) == 3, 'Should return all tiers'



"""
Tier helper and calculation classes with no model dependency.
"""
from collections import namedtuple

from django.utils import timezone
from django.utils.timesince import timeuntil

TierTuple = namedtuple('TierTuple', ['id', 'name'])


class TierInfo:
    """
    Tier info and calculator class.

    TODO: Move into the Site Configuration Client package.
    """

    TRIAL = TierTuple('trial', 'Trial')  # Expires in 30 days
    BASIC = TierTuple('basic', 'Basic')
    PRO = TierTuple('pro', 'Professional')
    PREMIUM = TierTuple('premium', 'Premium')

    TIERS = (
        TRIAL,
        BASIC,
        PRO,
        PREMIUM,
    )

    def __init__(self, tier, subscription_ends, always_active):
        self.tier = tier
        self.subscription_ends = subscription_ends
        self.always_active = always_active

    def has_subscription_ended(self, now=None):
        """Helper function that checks whether a subscription has expired"""
        if self.always_active:
            return False

        if not now:
            now = timezone.now()

        return now > self.subscription_ends

    def should_show_expiration_warning(self):
        """Decide if expiration warning is needed."""
        if self.always_active:
            return False

        return self.tier == self.TRIAL.id

    def time_til_expiration(self, now=None):
        """Pretty prints time left til expiration"""
        if self.always_active:
            return False

        if not now:
            now = timezone.now()

        return timeuntil(self.subscription_ends, now)



from django.conf import settings as django_settings


class Settings:
    """
    Utility class to get the settings
    """

    def organization_model(self):
        return getattr(django_settings, 'TIERS_ORGANIZATION_MODEL', 'organizations.Organization')

    def expired_redirect_url(self):
        return getattr(django_settings, 'TIERS_EXPIRED_REDIRECT_URL', None)

    def organization_tier_getter_name(self):
        return getattr(django_settings, 'TIERS_ORGANIZATION_TIER_GETTER_NAME', None)

    def redirect_white_list(self):
        return getattr(django_settings, 'TIERS_REDIRECT_WHITELIST', [
            '/admin'
        ])


settings = Settings()



# -*- coding: utf-8 -*-


from django.contrib import admin
from django.contrib.admin import SimpleListFilter
from django.db.models import Q
from django.utils import timezone

from .models import Tier


def make_exempt(modeladmin, request, queryset):
    queryset.update(tier_enforcement_exempt=True)


make_exempt.short_description = "Exempt from tier enforcement"


class ActiveTierFilter(SimpleListFilter):
    """
    Admin filter for tier active/inactive status.

    TODO: Add support for graceperiod
    """

    title = 'Tier status'
    parameter_name = 'tier_status'

    STATUS_ACTIVE = 'active'
    STATUS_INACTIVE = 'inactive'

    def lookups(self, request, model_admin):
        return [
            [self.STATUS_ACTIVE, 'Active Tiers'],
            [self.STATUS_INACTIVE, 'Inactive Tiers'],
        ]

    def queryset(self, request, queryset):
        active_tiers_filter = Q(tier_enforcement_exempt=True) | Q(tier_expires_at__gte=timezone.now())
        if self.value() == self.STATUS_ACTIVE:
            queryset = queryset.filter(active_tiers_filter)
        elif self.value() == self.STATUS_INACTIVE:
            queryset = queryset.exclude(active_tiers_filter)
        return queryset


class TierAdmin(admin.ModelAdmin):
    list_display = ('organization', 'get_microsites', 'name', 'tier_expires_at', 'tier_enforcement_exempt',)
    search_fields = ['organization__name', 'name']
    list_filter = ['name', ActiveTierFilter, 'tier_expires_at', 'tier_enforcement_exempt']
    actions = [make_exempt]

    def get_microsites(self, obj):
        return "\n".join(obj.organization.microsites.values_list('subdomain', flat=True))
    get_microsites.short_description = 'Microsite'
    get_microsites.admin_order_field = 'organization__microsites__subdomain'


admin.site.register(Tier, TierAdmin)



from datetime import timedelta

from django.utils import timezone
from django.db import models

from model_utils.models import TimeStampedModel
from model_utils import Choices


from .app_settings import settings
from .tier_info import TierInfo


def set_default_expiration(now=None):
    if not now:
        now = timezone.now()

    return now + timedelta(days=30)


class Tier(TimeStampedModel):
    TIERS = Choices(*[
        (tier.id, tier.id.upper(), tier.name)
        for tier in TierInfo.TIERS
    ])

    name = models.CharField(
        max_length=255,
        choices=TIERS,
        default=TIERS.TRIAL)
    organization = models.OneToOneField(
        settings.organization_model(),
        related_name='tier',
        null=True,
        blank=True,
        on_delete=models.DO_NOTHING)
    tier_enforcement_exempt = models.BooleanField(default=False)
    tier_enforcement_grace_period = models.PositiveIntegerField(
        # TODO: Remove deprecated field https://github.com/appsembler/django-tiers/issues/36
        default=14,
    )
    tier_expires_at = models.DateTimeField(
        default=set_default_expiration)

    def __str__(self):
        return 'Tier <org: {0} at tier: {1}>'.format(self.organization.name, self.name)

    class Meta:
        app_label = 'tiers'

    def get_tier_info(self):
        return TierInfo(
            tier=self.name,
            subscription_ends=self.tier_expires_at,
            always_active=self.tier_enforcement_exempt,
        )

    def has_tier_expired(self, now=None):
        """Helper function that checks whether a tier has expired"""
        return self.get_tier_info().has_subscription_ended(now=now)

    def time_til_tier_expires(self, now=None):
        """Pretty prints time left til expiration"""
        return self.get_tier_info().time_til_expiration(now=now)
